
/***************************************************
* C source with CCS
*  File Name: 20201025_K474_MK316C_accl_sensor.c
*  Description: check potentiometer and ACCL sensor out
*  , and turn on yellow LED and relay
*  COPYRIGHT 2020 MYCOMKITS.COM, owned by CNET LIMITED
***************************************************/
// include header file
#include   <12f1572.h>
#DEVICE ADC=10

// settings
#fuses   INTRC_IO, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOLVP, NOPROTECT, NODEBUG
#fuses   NOCLKOUT, NOWRT, PLL_SW, NOSTVREN
#use delay(CLOCK = 4000000)

//
#define RELAY PIN_A4
#define LED PIN_A5
#define START PIN_A3

//
#byte WPUA = 0x20C
#byte OPTION_REG = 0x95

//

long value_setting=256;
long value_sensor=256;
signed LONG value_x=0;
signed LONG value_y=0;
signed LONG Xoffset=0;   //x offset,  -93 to +93 at 1g
signed LONG Yoffset=0;   //y offset,  -93 to +93 at 1g
long value_sx[16];
long value_sy[16];
int hyste = 0;
int start_delay =40;

// prototyping
#SEPARATE

void get_sensor();

#SEPARATE

void get_setting();

#SEPARATE

void initializing();

#SEPARATE

void check_adc();

// main
void main()
{
   initializing (); //ADC port initialize
   output_low (PIN_A4); //active high, Relay off
   output_high (PIN_A5); //active low, LED off
   //calibration
   delay_ms (2000);

   //
   check_adc ();

   //
   Xoffset = value_x; //
   Yoffset = value_y; //
   //
   output_low (PIN_A5); // LED on
   delay_ms (200);
   output_high (PIN_A5); // LED off
   delay_ms (200);
   output_low (PIN_A5); // LED on
   delay_ms (200);
   output_high (PIN_A5); // LED off
   // main loop
   WHILE (1)
   {
      IF (input (PIN_A3) == 1)
      {
         delay_ms (50); //check it again FOR chattaling
         IF (input (PIN_A3) == 1)
         {
            //
            get_setting ();
            get_sensor ();

            //
            IF (start_delay > 0) start_delay = start_delay - 1;

            ELSE
            {
               start_delay = 0;

               IF (value_sensor > value_setting * 0.9)
               {
                  output_low (PIN_A5); // LED on
               }

               ELSE
               {
                  output_high (PIN_A5); // LED off
               }

               IF (value_sensor > value_setting)
               {
                  IF (hyste == 0)
                  {
                     output_high (PIN_A4); // Relay on
                     hyste = 50;
                  }
               }

               else IF (hyste == 0)
               {
                  output_low (PIN_A4); // Relay off
                  hyste = 0;
               }
            }
         }
      }
   }
}

//
#SEPARATE

void get_setting()
{
   set_adc_channel (0); //must wait 65u
   delay_us (65);
   value_setting = read_adc (); // 0 to 1023, 0.0V to 3.3V
   value_setting = 16 * value_setting; // 0 to 16368 at 1g
}

//
#SEPARATE

void get_sensor()
{
   FLOAT value_sensorx = 256;
   FLOAT value_sensory = 256;

   // moving average FOR X axis and Y or Z
   // A1 shows X axis, A2 shows Z or Y axis
   // moving average
   /*
   INT i;
   FLOAT value_sensorx = 256;
   FLOAT value_sensory = 256;
   FOR (i = 15; i>0; i--)
   {
      value_sx[i] = value_sx[i - 1];
      value_sy[i] = value_sy[i - 1];
   }

   */
   check_adc ();

   //
   value_x = value_x - Xoffset;
   value_y = value_y - Yoffset;
   value_sensorx = value_x;
   value_sensory = value_y;

   /*
   value_sx[0] = value_sensorx * value_sensorx; // 0 to 8100 at 1g
   value_sy[0] = value_sensory * value_sensory; // 0 to 8100 at 1g
   // summing
   value_sensorx = 0;
   value_sensory = 0;
   FOR (i = 0; i < 16; i++)
   {
      value_sensorx = value_sensorx + value_sx[i]; // 0 to 129600
      value_sensory = value_sensory + value_sy[i]; // 0 to 129600
   }

   value_sensorx = value_sensorx / 16; // 0 to 8100 at 1g
   value_sensory = value_sensory / 16; // 0 to 8100 at 1g
   */
   value_sensorx = value_sensorx * value_sensorx; // 0 to 8100 at 1g
   value_sensory = value_sensory * value_sensory; // 0 to 8100 at 1g
   value_sensor = value_sensorx + value_sensory; // 0 to 16200 at 1g
}

// system intializing
#SEPARATE

void initializing()
{
   SET_TRIS_A (0x0F); //A0 to 3 are input, 4, 5 are output
   // A / D converter initialize AN1 and AN2
   setup_adc_ports (sAN0|sAN1|sAN2|VSS_VDD); // gnd to 5v
   setup_adc (ADC_CLOCK_DIV_8); // 2usec
   // timer 1 intialize FOR hysteresis count down
   setup_timer_1 (T1_INTERNAL|T1_DIV_BY_8);
   set_timer1 (0xF4F0); // timer 1 setting almost 5msec
   //
   OPTION_REG = OPTION_REG&0x7F; // enable pull up
   WPUA = 0x08;

   //
   enable_interrupts (INT_TIMER1); // timer 1 enable
   enable_interrupts (GLOBAL); // global enable
}

// timer 1 inturruption hysteresis count down
#INT_timer1

void isr1(VOID)
{
   set_timer1 (0xF4F0); // timer 1 almost 5msec
   IF (hyste > 1) hyste = hyste - 1;
   ELSE hyste = 0;
}

//
#SEPARATE

void check_adc()
{
   set_adc_channel (1); //must wait 65u
   delay_us (65);
   value_x = read_adc (); // convert it into floating, 0 to 1023
   value_x = value_x - 511; //center 1.65V, - 93 to 93, 1.35V to 1.95V at 1g
   set_adc_channel (2); //must wait 65u
   delay_us (65);
   value_y = read_adc (); // convert it into floating, 0 to 1023
   value_y = value_y - 511; //center 1.65V, - 93 to 93, 1.35V to 1.95V at 1g
}

